"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalDeltaConnectionServer = void 0;
const server_lambdas_1 = require("@fluidframework/server-lambdas");
const server_memory_orderer_1 = require("@fluidframework/server-memory-orderer");
const server_services_client_1 = require("@fluidframework/server-services-client");
const server_services_core_1 = require("@fluidframework/server-services-core");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const server_test_utils_1 = require("@fluidframework/server-test-utils");
const localWebSocketServer_1 = require("./localWebSocketServer");
const localOrdererManager_1 = require("./localOrdererManager");
/**
 * Implementation of local delta connection server.
 */
class LocalDeltaConnectionServer {
    constructor(webSocketServer, databaseManager, ordererManager, testDbFactory, documentStorage, logger) {
        this.webSocketServer = webSocketServer;
        this.databaseManager = databaseManager;
        this.ordererManager = ordererManager;
        this.testDbFactory = testDbFactory;
        this.documentStorage = documentStorage;
        this.logger = logger;
    }
    /**
     * Creates and returns a local delta connection server.
     */
    static create(testDbFactory = new server_test_utils_1.TestDbFactory({}), serviceConfiguration) {
        if (!server_services_telemetry_1.Lumberjack.isSetupCompleted()) {
            server_services_telemetry_1.Lumberjack.setup([new server_services_telemetry_1.TestEngine1()]);
        }
        const nodesCollectionName = "nodes";
        const documentsCollectionName = "documents";
        const deltasCollectionName = "deltas";
        const scribeDeltasCollectionName = "scribeDeltas";
        const pubsub = new server_memory_orderer_1.PubSub();
        const webSocketServer = new localWebSocketServer_1.LocalWebSocketServer(pubsub);
        const mongoManager = new server_services_core_1.MongoManager(testDbFactory);
        const testTenantManager = new server_test_utils_1.TestTenantManager(undefined, undefined, testDbFactory.testDatabase);
        const databaseManager = new server_services_core_1.MongoDatabaseManager(false, mongoManager, null, nodesCollectionName, documentsCollectionName, deltasCollectionName, scribeDeltasCollectionName);
        const testStorage = new server_test_utils_1.TestDocumentStorage(databaseManager, testTenantManager);
        const logger = server_test_utils_1.DebugLogger.create("fluid-server:LocalDeltaConnectionServer");
        const ordererManager = new localOrdererManager_1.LocalOrdererManager(testStorage, databaseManager, testTenantManager, new server_services_core_1.EmptyTaskMessageSender(), {}, server_services_client_1.generateToken, async () => new server_test_utils_1.TestHistorian(testDbFactory.testDatabase), logger, serviceConfiguration, pubsub);
        (0, server_lambdas_1.configureWebSocketServices)(webSocketServer, ordererManager, testTenantManager, testStorage, new server_test_utils_1.TestClientManager(), new server_services_core_1.DefaultMetricClient(), logger);
        return new LocalDeltaConnectionServer(webSocketServer, databaseManager, ordererManager, testDbFactory, testStorage, logger);
    }
    async close() {
        await this.webSocketServer.close();
        await this.ordererManager.close();
    }
    /**
     * Returns true if there are any received ops that are not yet ordered.
     */
    async hasPendingWork() {
        return this.ordererManager.hasPendingWork();
    }
    connectWebSocket(tenantId, documentId, token, client, protocolVersions) {
        const socket = this.webSocketServer.createConnection();
        const connectMessage = {
            client,
            id: documentId,
            mode: client.mode,
            tenantId,
            token,
            versions: protocolVersions,
        };
        const connectedP = new Promise((resolve, reject) => {
            // Listen for ops sent before we receive a response to connect_document
            const queuedMessages = [];
            const queuedSignals = [];
            const earlyOpHandler = (docId, msgs) => {
                this.logger.info(`Queued early ops: ${msgs.length}`);
                server_services_telemetry_1.Lumberjack.info(`Queued early ops: ${msgs.length}`);
                queuedMessages.push(...msgs);
            };
            socket.on("op", earlyOpHandler);
            const earlySignalHandler = (msg) => {
                this.logger.info("Queued early signals");
                server_services_telemetry_1.Lumberjack.info("Queued early signals");
                queuedSignals.push(msg);
            };
            socket.on("signal", earlySignalHandler);
            // Listen for connection issues
            socket.on("connect_error", (error) => {
                reject(error);
            });
            socket.on("connect_document_success", (response) => {
                socket.removeListener("op", earlyOpHandler);
                socket.removeListener("signal", earlySignalHandler);
                if (queuedMessages.length > 0) {
                    // Some messages were queued.
                    // add them to the list of initialMessages to be processed
                    response.initialMessages.push(...queuedMessages);
                    response.initialMessages.sort((a, b) => a.sequenceNumber - b.sequenceNumber);
                }
                if (queuedSignals.length > 0) {
                    // Some signals were queued.
                    // add them to the list of initialSignals to be processed
                    response.initialSignals.push(...queuedSignals);
                }
                resolve(response);
            });
            socket.on("connect_document_error", reject);
            socket.emit("connect_document", connectMessage);
        });
        return [socket, connectedP];
    }
}
exports.LocalDeltaConnectionServer = LocalDeltaConnectionServer;
//# sourceMappingURL=localDeltaConnectionServer.js.map