/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { configureWebSocketServices } from "@fluidframework/server-lambdas";
import { PubSub } from "@fluidframework/server-memory-orderer";
import { generateToken } from "@fluidframework/server-services-client";
import { DefaultMetricClient, EmptyTaskMessageSender, MongoDatabaseManager, MongoManager, } from "@fluidframework/server-services-core";
import { Lumberjack, TestEngine1 } from "@fluidframework/server-services-telemetry";
import { DebugLogger, TestClientManager, TestDbFactory, TestDocumentStorage, TestHistorian, TestTenantManager, } from "@fluidframework/server-test-utils";
import { LocalWebSocketServer } from "./localWebSocketServer";
import { LocalOrdererManager } from "./localOrdererManager";
/**
 * Implementation of local delta connection server.
 */
export class LocalDeltaConnectionServer {
    constructor(webSocketServer, databaseManager, ordererManager, testDbFactory, documentStorage, logger) {
        this.webSocketServer = webSocketServer;
        this.databaseManager = databaseManager;
        this.ordererManager = ordererManager;
        this.testDbFactory = testDbFactory;
        this.documentStorage = documentStorage;
        this.logger = logger;
    }
    /**
     * Creates and returns a local delta connection server.
     */
    static create(testDbFactory = new TestDbFactory({}), serviceConfiguration) {
        if (!Lumberjack.isSetupCompleted()) {
            Lumberjack.setup([new TestEngine1()]);
        }
        const nodesCollectionName = "nodes";
        const documentsCollectionName = "documents";
        const deltasCollectionName = "deltas";
        const scribeDeltasCollectionName = "scribeDeltas";
        const pubsub = new PubSub();
        const webSocketServer = new LocalWebSocketServer(pubsub);
        const mongoManager = new MongoManager(testDbFactory);
        const testTenantManager = new TestTenantManager(undefined, undefined, testDbFactory.testDatabase);
        const databaseManager = new MongoDatabaseManager(false, mongoManager, null, nodesCollectionName, documentsCollectionName, deltasCollectionName, scribeDeltasCollectionName);
        const testStorage = new TestDocumentStorage(databaseManager, testTenantManager);
        const logger = DebugLogger.create("fluid-server:LocalDeltaConnectionServer");
        const ordererManager = new LocalOrdererManager(testStorage, databaseManager, testTenantManager, new EmptyTaskMessageSender(), {}, generateToken, async () => new TestHistorian(testDbFactory.testDatabase), logger, serviceConfiguration, pubsub);
        configureWebSocketServices(webSocketServer, ordererManager, testTenantManager, testStorage, new TestClientManager(), new DefaultMetricClient(), logger);
        return new LocalDeltaConnectionServer(webSocketServer, databaseManager, ordererManager, testDbFactory, testStorage, logger);
    }
    async close() {
        await this.webSocketServer.close();
        await this.ordererManager.close();
    }
    /**
     * Returns true if there are any received ops that are not yet ordered.
     */
    async hasPendingWork() {
        return this.ordererManager.hasPendingWork();
    }
    connectWebSocket(tenantId, documentId, token, client, protocolVersions) {
        const socket = this.webSocketServer.createConnection();
        const connectMessage = {
            client,
            id: documentId,
            mode: client.mode,
            tenantId,
            token,
            versions: protocolVersions,
        };
        const connectedP = new Promise((resolve, reject) => {
            // Listen for ops sent before we receive a response to connect_document
            const queuedMessages = [];
            const queuedSignals = [];
            const earlyOpHandler = (docId, msgs) => {
                this.logger.info(`Queued early ops: ${msgs.length}`);
                Lumberjack.info(`Queued early ops: ${msgs.length}`);
                queuedMessages.push(...msgs);
            };
            socket.on("op", earlyOpHandler);
            const earlySignalHandler = (msg) => {
                this.logger.info("Queued early signals");
                Lumberjack.info("Queued early signals");
                queuedSignals.push(msg);
            };
            socket.on("signal", earlySignalHandler);
            // Listen for connection issues
            socket.on("connect_error", (error) => {
                reject(error);
            });
            socket.on("connect_document_success", (response) => {
                socket.removeListener("op", earlyOpHandler);
                socket.removeListener("signal", earlySignalHandler);
                if (queuedMessages.length > 0) {
                    // Some messages were queued.
                    // add them to the list of initialMessages to be processed
                    response.initialMessages.push(...queuedMessages);
                    response.initialMessages.sort((a, b) => a.sequenceNumber - b.sequenceNumber);
                }
                if (queuedSignals.length > 0) {
                    // Some signals were queued.
                    // add them to the list of initialSignals to be processed
                    response.initialSignals.push(...queuedSignals);
                }
                resolve(response);
            });
            socket.on("connect_document_error", reject);
            socket.emit("connect_document", connectMessage);
        });
        return [socket, connectedP];
    }
}
//# sourceMappingURL=localDeltaConnectionServer.js.map