/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { LocalOrderer } from "@fluidframework/server-memory-orderer";
import { GitManager } from "@fluidframework/server-services-client";
export class LocalOrdererManager {
    constructor(storage, databaseManager, tenantManager, taskMessageSender, permission, // Can probably remove
    tokenGenerator, createHistorian, logger, serviceConfiguration, pubsub) {
        this.storage = storage;
        this.databaseManager = databaseManager;
        this.tenantManager = tenantManager;
        this.taskMessageSender = taskMessageSender;
        this.permission = permission;
        this.tokenGenerator = tokenGenerator;
        this.createHistorian = createHistorian;
        this.logger = logger;
        this.serviceConfiguration = serviceConfiguration;
        this.pubsub = pubsub;
        this.map = new Map();
    }
    /**
     * Closes all local orderers
     */
    async close() {
        await Promise.all(Array.from(this.map.values()).map(async (orderer) => (await orderer).close()));
        this.map.clear();
    }
    /**
     * Returns true if there are any received ops that are not yet ordered.
     */
    async hasPendingWork() {
        return Promise.all(this.map.values()).then((orderers) => {
            for (const orderer of orderers) {
                // We know that it ia LocalOrderer, break the abstraction
                if (orderer.hasPendingWork()) {
                    return true;
                }
            }
            return false;
        });
    }
    async getOrderer(tenantId, documentId) {
        const key = `${tenantId}/${documentId}`;
        if (!this.map.has(key)) {
            const orderer = this.createLocalOrderer(tenantId, documentId);
            this.map.set(key, orderer);
        }
        return this.map.get(key);
    }
    async createLocalOrderer(tenantId, documentId) {
        const historian = await this.createHistorian(tenantId);
        const gitManager = new GitManager(historian);
        const orderer = await LocalOrderer.load(this.storage, this.databaseManager, tenantId, documentId, this.taskMessageSender, this.tenantManager, this.permission, this.tokenGenerator, this.logger, gitManager, undefined /* ILocalOrdererSetup */, this.pubsub, undefined /* broadcasterContext */, undefined /* scriptoriumContext */, undefined /* foremanContext */, undefined /* scribeContext */, undefined /* deliContext */, undefined /* moiraContext */, this.serviceConfiguration);
        const lambdas = [
            orderer.broadcasterLambda,
            orderer.deliLambda,
            orderer.foremanLambda,
            orderer.scribeLambda,
            orderer.scriptoriumLambda,
        ];
        await Promise.all(lambdas.map(async (l) => {
            if (l.state === "created") {
                return new Promise((resolve) => l.once("started", () => resolve()));
            }
        }));
        return orderer;
    }
}
//# sourceMappingURL=localOrdererManager.js.map